@@ -1,24 +1,20 @@
 package App::bif::show::entity;
 use strict;
 use warnings;
-use App::bif::Context;
-use App::bif::show;
+use parent 'App::bif::show';
 
-our $VERSION = '0.1.0_26';
+our $VERSION = '0.1.0_27';
 
 sub run {
-    my $ctx = App::bif::Context->new(shift);
-    my $db  = $ctx->db;
-
-    $ctx->{id} = $ctx->uuid2id( $ctx->{id} );
-
-    App::bif::show::_init;
+    my $self = __PACKAGE__->new(shift);
+    my $db   = $self->db;
+    $self->{id} = $self->uuid2id( $self->{id} );
 
     my @data;
 
     DBIx::ThinSQL->import(qw/sum case coalesce concat qv/);
 
-    my $ref = $db->xhash(
+    my $ref = $db->xhashref(
         select => [
             't.id',
             'substr(t.uuid,1,8) as uuid',
@@ -47,31 +43,28 @@ sub run {
         on         => 'h.id = e.hub_id',
         inner_join => 'hub_repos hr',
         on         => 'hr.id = h.default_repo_id',
-        where      => { 'e.id' => $ctx->{id} },
+        where      => { 'e.id' => $self->{id} },
     );
 
-    return $ctx->err( 'EntityNotFound', "entity not found: $ctx->{id}" )
+    return $self->err( 'EntityNotFound', "entity not found: $self->{id}" )
       unless $ref;
 
-    my $bold = Term::ANSIColor::color('bold');
+    $self->init;
+    my ($bold) = $self->colours('bold');
 
     push(
         @data,
-        App::bif::show::_header( $bold . 'Entity', $bold . $ref->{name} ),
-        App::bif::show::_header( '  UUID',         $ref->{uuid} ),
-        App::bif::show::_header(
-            '  Hub', $ref->{hub_name}, $ref->{hub_location}
-        ),
-        App::bif::show::_header(
-            '  Updated',
-            App::bif::show::_new_ago( $ref->{mtime}, $ref->{mtimetz} )
+        $self->header( '  UUID', $ref->{uuid} ),
+        $self->header( '  Hub', $ref->{hub_name}, $ref->{hub_location} ),
+        $self->header(
+            '  Updated', $self->ago( $ref->{mtime}, $ref->{mtimetz} )
         ),
     );
 
-    push( @data, App::bif::show::_header( '  Contact', $ref->{contact} ), )
+    push( @data, $self->header( '  Contact', $ref->{contact} ), )
       if $ref->{other_contact};
 
-    my @methods = $db->xhashes(
+    my @methods = $db->xhashrefs(
         select => [
             'ecm.method', 'ecm.mvalue',
             'ecm.id = e.default_contact_method_id AS preferred',
@@ -79,13 +72,13 @@ sub run {
         from       => 'entities e',
         inner_join => 'entity_contact_methods ecm',
         on         => 'ecm.entity_id = e.id',
-        where      => { 'e.id' => $ctx->{id} },
+        where      => { 'e.id' => $self->{id} },
         order_by   => [qw/ ecm.method ecm.mvalue /],
     );
 
     push(
         @data,
-        App::bif::show::_header(
+        $self->header(
             '  '
               . ( $ref->{other_contact} ? '  ' : '' )
               . ucfirst( $_->{method} ),
@@ -94,11 +87,12 @@ sub run {
         ),
     ) for @methods;
 
-    $ctx->start_pager;
-    print $ctx->render_table( 'l  l', undef, \@data );
-    $ctx->end_pager;
+    $self->start_pager;
+    print $self->render_table( 'l  l',
+        $self->header( 'Entity', $ref->{name} ), \@data );
+    $self->end_pager;
 
-    return $ctx->ok( 'ShowEntity', \@data );
+    return $self->ok( 'ShowEntity', \@data );
 }
 
 1;
@@ -110,7 +104,7 @@ bif-show-entity - display a entity's current status
 
 =head1 VERSION
 
-0.1.0_26 (2014-07-23)
+0.1.0_27 (2014-09-10)
 
 =head1 SYNOPSIS
 
@@ -133,7 +127,7 @@ An entity ID. Required.
 
 Display a more verbose version of the current status.
 
-=item --uuid, -u
+=item --uuid, -U
 
 Lookup the topic using ID as a UUID string instead of a topic integer.
 
