@@ -6,10 +6,11 @@ use DBIx::ThinSQL qw/qv/;
 use Log::Any '$log';
 use Role::Basic;
 
-our $VERSION = '0.1.0_26';
+our $VERSION = '0.1.0_27';
 
 my %import_functions = (
     NEW => {
+        topic                 => 'func_import_topic',
         entity                => 'func_import_entity',
         entity_contact_method => 'func_import_entity_contact_method',
         identity              => 'func_import_identity',
@@ -35,6 +36,7 @@ my %import_functions = (
         hub_repo              => 'func_import_hub_repo_delta',
         task                  => 'func_import_task_delta',
         task_status           => 'func_import_task_status_delta',
+        update                => 'func_import_update_delta',
     },
     QUIT   => {},
     CANCEL => {},
@@ -75,7 +77,7 @@ sub recv_hub_deltas {
         if ( $action eq 'NEW' and $type eq 'update' ) {
             $ucount = delete $ref->{ucount};
 
-            ($id) = $db->xarray(
+            $id = $db->xval(
                 select => 'u.id',
                 from   => 'updates u',
                 where  => { 'u.uuid' => $ref->{uuid} },
@@ -142,7 +144,7 @@ sub real_sync_hub {
 
     $on_update->( 'matching: ' . $prefix2 ) if $on_update;
 
-    my @refs = $db->xarrays(
+    my @refs = $db->xarrayrefs(
         select => [qw/rm.prefix rm.hash/],
         from   => 'hub_related_updates_merkle rm',
         where =>
@@ -174,7 +176,7 @@ sub real_sync_hub {
         my @where;
         foreach my $miss (@missing) {
             push( @where, ' OR ' ) if @where;
-            push( @where, "u.prefix LIKE ", qv( $miss . '%' ) ),;
+            push( @where, "u.uuid LIKE ", qv( $miss . '%' ) ),;
         }
 
         $self->db->xdo(
@@ -205,7 +207,7 @@ sub real_transfer_hub_updates {
     my $tmp  = $self->temp_table;
 
     my $send = async {
-        my ($total) = $self->db->xarray(
+        my $total = $self->db->xval(
             select => 'COALESCE(sum(t.ucount), 0)',
             from   => "$tmp t",
         );
@@ -220,16 +222,19 @@ sub real_transfer_hub_updates {
                 'u.mtime',               'u.mtimetz',
                 'u.author',              'u.email',
                 'u.lang',                'u.message',
-                'u.ucount',
+                'u.action',              'u.ucount',
             ],
             from       => "$tmp tmp",
             inner_join => 'updates u',
             on         => 'u.id = tmp.id',
-            inner_join => 'topics t',
-            on         => 't.id = u.identity_id',
-            left_join  => 'updates p',
-            on         => 'p.id = u.parent_id',
-            order_by   => 'u.id ASC',
+            left_join  => 'topics t',
+
+            # Don't fetch the identity_uuid for the first identity
+            # update
+            on        => 't.id = u.identity_id AND t.first_update_id != u.id',
+            left_join => 'updates p',
+            on        => 'p.id = u.parent_id',
+            order_by  => 'u.id ASC',
         );
 
         $update_list->execute;
@@ -257,7 +262,7 @@ sub real_export_hub {
     my $self = shift;
     my $id   = shift;
 
-    my ($total) = $self->db->xarray(
+    my $total = $self->db->xval(
         select     => 'sum(u.ucount)',
         from       => 'hub_related_updates rru',
         inner_join => 'updates u',
@@ -270,22 +275,25 @@ sub real_export_hub {
 
     my $sth = $self->db->xprepare(
         select => [
-            'updates.id',                  'updates.uuid',
-            'parents.uuid AS parent_uuid', 't.uuid AS identity_uuid',
-            'updates.mtime',               'updates.mtimetz',
-            'updates.author',              'updates.email',
-            'updates.lang',                'updates.message',
-            'updates.ucount',
+            'u.id',                  'u.uuid',
+            'p.uuid AS parent_uuid', 't.uuid AS identity_uuid',
+            'u.mtime',               'u.mtimetz',
+            'u.author',              'u.email',
+            'u.lang',                'u.message',
+            'u.action',              'u.ucount',
         ],
-        from       => 'hub_related_updates AS rru',
-        inner_join => 'updates',
-        on         => 'updates.id = rru.update_id',
-        inner_join => 'topics t',
-        on         => 't.id = updates.identity_id',
-        left_join  => 'updates AS parents',
-        on         => 'parents.id = updates.parent_id',
-        where      => { 'rru.hub_id' => $id },
-        order_by   => 'updates.id ASC',
+        from       => 'hub_related_updates rru',
+        inner_join => 'updates u',
+        on         => 'u.id = rru.update_id',
+        left_join  => 'topics t',
+
+        # Don't fetch the identity_uuid for the first identity
+        # update
+        on        => 't.id = u.identity_id AND t.first_update_id != u.id',
+        left_join => 'updates p',
+        on        => 'p.id = u.parent_id',
+        where     => { 'rru.hub_id' => $id },
+        order_by  => 'u.id ASC',
     );
 
     $sth->execute;
